@@ -4,7 +4,7 @@ import sys
 import json as j
 from ruamel.yaml import YAML
 import click
-from .config import get_config, write_configfile
+from .config import get_config, write_configfile, DEFAULT_CONFIGFILE_NAME
 from .inspect import pprint_tapshapes
 from .csvreader import csvreader
 from .loggers import stderr_logger
@@ -16,7 +16,7 @@ from .utils import expand_uri_prefixes
 
 
 @click.group()
-@click.version_option("0.2.2", help="Show version and exit")
+@click.version_option("0.2.3", help="Show version and exit")
 @click.help_option(help="Show help and exit")
 @click.pass_context
 def cli(context):
@@ -24,7 +24,7 @@ def cli(context):
 
 
 @cli.command()
-@click.argument("csvfile_name", type=click.File(mode="r", encoding="utf-8-sig"))
+@click.argument("csvfile_obj", type=click.File(mode="r", encoding="utf-8-sig"))
 @click.option(
     "--configfile", type=click.Path(exists=True), help="Pathname of configuration file."
 )
@@ -38,16 +38,15 @@ def cli(context):
 @click.option("--yaml", is_flag=True, help="Print YAML to stdout.")
 @click.help_option(help="Show help and exit")
 @click.pass_context
-def generate(context, csvfile_name, configfile, expand_prefixes, warnings, json, yaml):
-    """Given CSV, generate text, JSON, or YAML, with warnings."""
+def generate(context, csvfile_obj, configfile, expand_prefixes, warnings, json, yaml):
+    """Generate normalized text, JSON, or YAML of CSV, with warnings."""
     # pylint: disable=too-many-locals,too-many-arguments
 
     config_dict = get_config(configfile)
-    csvreader_output = csvreader(csvfile_name, config_dict)
-    tapshapes_dict = csvreader_output[0]
+    csvreader_output = csvreader(csvfile_obj, config_dict)
+    tapshapes_dict, warnings_dict = csvreader_output
     if expand_prefixes:
         tapshapes_dict = expand_uri_prefixes(tapshapes_dict, config_dict)
-    warnings_dict = csvreader_output[1]
 
     if json and yaml:
         # Quick fix for mutually exclusive options, a better fix in future.
@@ -66,7 +65,7 @@ def generate(context, csvfile_name, configfile, expand_prefixes, warnings, json,
 
     # pylint: disable=logging-fstring-interpolation
     if not (json or yaml):
-        pprint_output = pprint_tapshapes(tapshapes_dict)
+        pprint_output = pprint_tapshapes(tapshapes_dict, config_dict)
         for line in pprint_output:
             print(line, file=sys.stdout)
         if warnings:
@@ -77,10 +76,13 @@ def generate(context, csvfile_name, configfile, expand_prefixes, warnings, json,
                     for warning in warn_list:
                         echo.warning(f"[{shapeid}/{elem}] {warning}")
 
+
 @cli.command()
 @click.argument("configfile", type=click.Path(), required=False)
 @click.help_option(help="Show help and exit")
 @click.pass_context
 def init(context, configfile):
-    """Write out starter config file [default: dctap.yml]"""
+    """Generate customizable configuration file [default: dctap.yml]."""
+    if not configfile:
+        configfile = DEFAULT_CONFIGFILE_NAME
     write_configfile(configfile)
