@@ -17,8 +17,8 @@ class TAPStatementConstraint:
 
     propertyID: str = ""
     propertyLabel: str = ""
-    mandatory: str = None
-    repeatable: str = None
+    mandatory: str = ""
+    repeatable: str = ""
     valueNodeType: str = ""
     valueDataType: str = ""
     valueConstraint: str = ""
@@ -26,83 +26,90 @@ class TAPStatementConstraint:
     valueShape: str = ""
     note: str = ""
     sc_warnings: dict = field(default_factory=dict)
+    extra_elements: dict = field(default_factory=dict)
 
-    def validate(self, config_dict):
-        """Validates specific fields."""
+    def normalize(self, settings):
+        """Normalizes specific fields."""
         # pylint: disable=attribute-defined-outside-init
-        self.config_dict = config_dict
-        self._elements_taking_IRIs_warn_if_not_IRIs()
-        self._mandatory_repeatable_have_supported_boolean_values()
+        self._warn_if_propertyID_or_valueDataType_not_IRIlike()
+        self._normalize_booleans_mandatory_repeatable()
         self._valueConstraintType_pattern_warn_if_valueConstraint_not_valid_regex()
-        self._valueConstraintType_iristem_warn_if_valueConstraint_not_is_not_an_IRI()
-        self._valueConstraintType_picklist_parse()
-        self._valueConstraintType_languageTag_parse()
+        self._valueConstraintType_pattern_warn_if_used_with_value_shape()
+        self._valueConstraintType_iristem_parse()
+        self._valueConstraintType_iristem_warn_if_list_items_not_IRIs()
+        self._valueConstraintType_languageTag_parse(settings)
         self._valueConstraintType_warn_if_used_without_valueConstraint()
         self._valueDataType_warn_if_used_with_valueNodeType_IRI()
-        self._valueNodeType_is_from_enumerated_list()
+        self._valueDataType_warn_if_valueNodeType_literal_used_with_any_valueShape()
+        self._valueConstraintType_picklist_parse(settings)
+        self._valueNodeType_is_from_enumerated_list(settings)
         return self
 
-    def _elements_taking_IRIs_warn_if_not_IRIs(self):
+    def _warn_if_propertyID_or_valueDataType_not_IRIlike(self):
         """@@@"""
         if not is_uri_or_prefixed_uri(self.propertyID):
             self.sc_warnings[
                 "propertyID"
             ] = f"{repr(self.propertyID)} is not an IRI or Compact IRI."
-        if self.valueShape:
-            if not is_uri_or_prefixed_uri(self.valueShape):
-                self.sc_warnings[
-                    "valueShape"
-                ] = f"{repr(self.valueShape)} is not an IRI or Compact IRI."
         if self.valueDataType:
             if not is_uri_or_prefixed_uri(self.valueDataType):
                 self.sc_warnings[
                     "valueDataType"
                 ] = f"{repr(self.valueDataType)} is not an IRI or Compact IRI."
 
-    def _mandatory_repeatable_have_supported_boolean_values(self):
+    def _normalize_booleans_mandatory_repeatable(self):
         """Booleans take true/false (case-insensitive) or 1/0, default None."""
 
         valid_values_for_true = ["true", "1"]
         valid_values_for_false = ["false", "0"]
-        valid_values = valid_values_for_true + valid_values_for_false + [None]
+        valid_values = valid_values_for_true + valid_values_for_false
 
         # pylint: disable=singleton-comparison
-        if self.mandatory != None:
+        if self.mandatory:
             # breakpoint(context=5)
             mand = self.mandatory.lower()
-            if mand not in valid_values and mand != "":
+            if mand not in valid_values:
                 self.sc_warnings[
                     "mandatory"
                 ] = f"{repr(self.mandatory)} is not a supported Boolean value."
             if mand in valid_values_for_true:
-                self.mandatory = True
+                self.mandatory = "True"
             elif mand in valid_values_for_false:
-                self.mandatory = False
+                self.mandatory = "False"
 
-        if self.repeatable != None:
+        if self.repeatable:
             # breakpoint(context=5)
             repeat = self.repeatable.lower()
-            if repeat not in valid_values and repeat != "":
+            if repeat not in valid_values:
                 self.sc_warnings[
                     "repeatable"
                 ] = f"{repr(self.repeatable)} is not a supported Boolean value."
             if repeat in valid_values_for_true:
-                self.repeatable = True
+                self.repeatable = "True"
             elif repeat in valid_values_for_false:
-                self.repeatable = False
+                self.repeatable = "False"
 
         return self
 
-    def _valueConstraintType_iristem_warn_if_valueConstraint_not_is_not_an_IRI(self):
-        """If valueConstraintType IRIStem, warn if valueConstraint not an IRI."""
+    def _valueConstraintType_iristem_parse(self):
+        """If valueConstraintType is Iristem, split valueConstraint on whitespace."""
         self.valueConstraintType = self.valueConstraintType.lower()
         if self.valueConstraintType == "iristem":
-            if not is_uri_or_prefixed_uri(self.valueConstraint):
-                self.sc_warnings["valueConstraint"] = (
-                    f"Value constraint type is {repr(self.valueConstraintType)}, but "
-                    f"{repr(self.valueConstraint)} does not look like an IRI or "
-                    "Compact IRI."
-                )
+            if self.valueConstraint:
+                self.valueConstraint = self.valueConstraint.split()
+        return self
+
+    def _valueConstraintType_iristem_warn_if_list_items_not_IRIs(self):
+        """If IRIStem, warn if valueConstraint list items do not look like IRIs."""
+        self.valueConstraintType = self.valueConstraintType.lower()
+        if self.valueConstraintType == "iristem":
+            for list_item in self.valueConstraint:
+                if not is_uri_or_prefixed_uri(list_item):
+                    self.sc_warnings["valueConstraint"] = (
+                        f"Value constraint type is {repr(self.valueConstraintType)}, "
+                        f"but {repr(list_item)} does not look like an IRI or "
+                        "Compact IRI."
+                    )
         return self
 
     def _valueConstraintType_pattern_warn_if_valueConstraint_not_valid_regex(self):
@@ -118,20 +125,25 @@ class TAPStatementConstraint:
                 )
         return self
 
-    def _valueConstraintType_languageTag_parse(self):
+    def _valueConstraintType_pattern_warn_if_used_with_value_shape(self):
+        """Regular expressions cannot conform to value shapes."""
+        self.valueConstraintType = self.valueConstraintType.lower()
+        if self.valueConstraintType == "pattern":
+            if self.valueShape:
+                self.sc_warnings["valueConstraintType"] = (
+                    f"Value constraint type "
+                    f"({repr(self.valueConstraintType)}) "
+                    "cannot conform to a value shape."
+                )
+
+    def _valueConstraintType_languageTag_parse(self, settings):
         """For valueConstraintType languageTag, splits valueConstraint on whitespace."""
         self.valueConstraintType = self.valueConstraintType.lower()
+        sep = settings.get("picklist_item_separator", " ")
         if self.valueConstraintType == "languagetag":
             if self.valueConstraint:
-                self.valueConstraint = self.valueConstraint.split()
-        return self
-
-    def _valueConstraintType_picklist_parse(self):
-        """If valueConstraintType is Picklist, splits valueConstraint on whitespace."""
-        self.valueConstraintType = self.valueConstraintType.lower()
-        if self.valueConstraintType == "picklist":
-            if self.valueConstraint:
-                self.valueConstraint = self.valueConstraint.split()
+                self.valueConstraint = self.valueConstraint.split(sep)
+                self.valueConstraint = [x.strip() for x in self.valueConstraint if x]
         return self
 
     def _valueConstraintType_warn_if_used_without_valueConstraint(self):
@@ -139,17 +151,27 @@ class TAPStatementConstraint:
         if self.valueConstraintType:
             if not self.valueConstraint:
                 self.sc_warnings["valueConstraint"] = (
-                    f"Value constraint type is {repr(self.valueConstraintType)}, but "
-                    f"value constraint is empty."
+                    f"Value constraint type "
+                    f"({repr(self.valueConstraintType)}) "
+                    "but no value constraint."
                 )
         return self
 
-    def _valueNodeType_is_from_enumerated_list(self):
+    def _valueConstraintType_picklist_parse(self, settings):
+        """If valueConstraintType is Picklist, split valueConstraint on whitespace."""
+        self.valueConstraintType = self.valueConstraintType.lower()
+        sep = settings.get("picklist_item_separator", " ")
+        if self.valueConstraintType == "picklist":
+            if self.valueConstraint:
+                self.valueConstraint = self.valueConstraint.split(sep)
+                self.valueConstraint = [x.strip() for x in self.valueConstraint if x]
+        return self
+
+    def _valueNodeType_is_from_enumerated_list(self, settings):
         """Take valueNodeType from configurable enumerated list, case-insensitive."""
-        if self.config_dict["value_node_types"]:
-            valid_types = [vnt.lower() for vnt in self.config_dict["value_node_types"]]
-        else:
-            valid_types = ["iri", "bnode", "literal"]
+        valid_types = ["iri", "bnode", "literal"]
+        if settings.get("value_node_types"):
+            valid_types += [vnt.lower() for vnt in settings["value_node_types"]]
         if self.valueNodeType:
             self.valueNodeType = self.valueNodeType.lower()  # normalize to lowercase
             if self.valueNodeType not in valid_types:
@@ -158,10 +180,29 @@ class TAPStatementConstraint:
                 ] = f"{repr(self.valueNodeType)} is not a valid node type."
         return self
 
+    def _valueDataType_warn_if_valueNodeType_literal_used_with_any_valueShape(self):
+        """Value with node type Literal cannot conform to a value shape."""
+        self.valueNodeType = self.valueNodeType.lower()
+        if self.valueShape:
+            if self.valueNodeType == "literal":
+                self.sc_warnings["valueDataType"] = (
+                    "Datatypes are only for literals, "
+                    "which cannot conform to a value shape."
+                )
+
+    def _valueDataType_warn_if_used_with_valueShape(self):
+        """Value with any datatype cannot conform to a value shape."""
+        if self.valueShape:
+            if self.valueDataType:
+                self.sc_warnings["valueDataType"] = (
+                    "Datatypes are only for literals, "
+                    "which cannot conform to a value shape."
+                )
+
     def _valueDataType_warn_if_used_with_valueNodeType_IRI(self):
-        """@@@"""
+        """Value with datatype implies Literal and cannot be node type IRI."""
         node_type = self.valueNodeType.lower()
-        if node_type in ('iri', 'uri', 'bnode'):
+        if node_type in ("iri", "uri", "bnode"):
             if self.valueDataType:
                 self.sc_warnings["valueDataType"] = (
                     f"Datatypes are only for literals, "
@@ -169,7 +210,7 @@ class TAPStatementConstraint:
                 )
 
     def get_warnings(self):
-        """Emit warnings dictionary self.sc_warnings, populated by validate() method."""
+        """Emit warnings dictionary self.sc_warnings, populated by normalize() method."""
         return dict(self.sc_warnings)
 
 
@@ -183,28 +224,20 @@ class TAPShape:
     shapeID: str = ""
     shapeLabel: str = ""
     sc_list: List[TAPStatementConstraint] = field(default_factory=list)
-#    statement_constraints: List[TAPStatementConstraint] = field(default_factory=list)
     sh_warnings: dict = field(default_factory=dict)
+    extra_elements: dict = field(default_factory=dict)
 
-    def validate(self, config_dict=None):
+    def normalize(self, settings):
         """Normalize values where required."""
-        self._normalize_default_shapeID(config_dict)
-        self._warn_if_shapeID_is_not_an_IRI()
+        self._normalize_default_shapeID(settings)
         return True
 
-    def _normalize_default_shapeID(self, config_dict=None):
-        """If shapeID not specified, sets default value from config."""
+    def _normalize_default_shapeID(self, settings):
+        """If shapeID not specified, looks first in config, else sets "default"."""
         if not self.shapeID:
-            self.shapeID = config_dict["default_shape_name"]
+            self.shapeID = settings.get("default_shape_identifier", "default")
         return self
 
-    def _warn_if_shapeID_is_not_an_IRI(self):
-        """@@@"""
-        if not is_uri_or_prefixed_uri(self.shapeID):
-            self.sh_warnings[
-                "shapeID"
-            ] = f"{repr(self.shapeID)} is not an IRI or Compact IRI."
-
     def get_warnings(self):
-        """Emit warnings dictionary self.sh_warnings, populated by validate() method."""
+        """Emit warnings dictionary self.sh_warnings, populated by normalize() method."""
         return dict(self.sh_warnings)
