@@ -15,6 +15,7 @@
 #
 import os
 import sys
+import dctap
 
 sys.path.insert(0, os.path.abspath("../dctap/"))
 
@@ -25,7 +26,7 @@ project = "dctap-python"
 copyright = "2021, DCMI"
 
 # The full version, including alpha/beta/rc tags
-release = "0.1"
+release = dctap.__version__
 
 # -- General configuration ---------------------------------------------------
 
